//
//  Context.cpp
//  BouncingBallWithOde
//

#include "Context.h"
#include "Evaluator.h"
#include <stdexcept>

boost::shared_ptr<Context> Context::get_instance()
{
        if ( !instance_ ) {
                instance_.reset(new Context());
        }
        return instance_;
}

boost::shared_ptr<Context> Context::instance_;

Context::Context()
        : context_()
        , script_node_()
        , mutex_() {}
        
void Context::initialize()
{
        boost::lock_guard<boost::mutex> lock(mutex_);
        XnStatus rc = context_.Init();
        Evaluator::check_status(rc);
}

void Context::set_global_mirror(bool flag)
{
        boost::lock_guard<boost::mutex> lock(mutex_);
        XnStatus rc = context_.SetGlobalMirror(flag);
        Evaluator::check_status(rc);
}

void Context::enumerate_production_trees(xn::NodeInfoList& nodes, XnProductionNodeType type)
{
        boost::lock_guard<boost::mutex> lock(mutex_);
        XnStatus rc = context_.EnumerateProductionTrees(type, NULL, nodes);
        if ( rc != XN_STATUS_OK ) {
                throw std::runtime_error(xnGetStatusString(rc));
        } else if ( nodes.Begin () == nodes.End() ) {
                throw std::runtime_error("devices not found.");
        }
        std::cout << "Success" << std::endl;
}

void Context::create_production_tree(xn::NodeInfo& info)
{
        boost::lock_guard<boost::mutex> lock(mutex_);
        XnStatus rc = context_.CreateProductionTree(info);
        Evaluator::check_status(rc);
}

void Context::enumerate_existing_nodes(xn::NodeInfoList& node_info_list)
{
        boost::lock_guard<boost::mutex> lock(mutex_);
        XnStatus rc = context_.EnumerateExistingNodes(node_info_list);
        Evaluator::check_status(rc);
}

void Context::wait_any_update_all()
{
        boost::lock_guard<boost::mutex> lock(mutex_);
        XnStatus rc = context_.WaitAnyUpdateAll();
        Evaluator::check_status(rc);
}

void Context::wait_and_update_all()
{
        boost::lock_guard<boost::mutex> lock(mutex_);
        XnStatus rc = context_.WaitAndUpdateAll();
        Evaluator::check_status(rc);
}

void Context::start_generating_all()
{
        boost::lock_guard<boost::mutex> lock(mutex_);
        XnStatus rc = context_.StartGeneratingAll();
        Evaluator::check_status(rc);
}
