//
//  Viewer.h
//  BouncingBallWithOde
//

#ifndef __OpenNiMultiViewer__Viewer__
#define __OpenNiMultiViewer__Viewer__

#include <boost/noncopyable.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/thread.hpp>
#include <boost/asio.hpp>
#include <boost/asio/steady_timer.hpp>
#include <string>
#include <XnTypes.h>
#include <ode/common.h>
#include "Device.h"
#include "PhysicalSimulator.h"

namespace xn
{
        class Context;
}

class Viewer : private boost::noncopyable
{
public:
        Viewer();
        
        int initialize(const XnMapOutputMode& map_mode, int index, const boost::shared_ptr<Device>& device);
        void set_display_callback(void(*display)());
        void display(void (*near_callback)(void* data, dGeomID o1, dGeomID o2));
        void on_key(unsigned char key, int /*x*/, int /*y*/);
        int get_id() const
        {
                return window_id_;
        }
        void near_callback(void* data, dGeomID o1, dGeomID o2);
private:
        // for more than one viewer
        static int viewer_number_;
        
        int index_;
        boost::shared_ptr<Device> device_;
        int width_;
        int height_;
        int window_id_;
                
        bool is_extracted_;
        bool stops_frame_rate_;
        bool displays_polygon_;
        bool displays_balls_;
        
        boost::shared_ptr<Device::TriangularMesh> mesh_;
        
        boost::mutex simulator_mutex_;
        boost::scoped_ptr<PhysicalSimulator> simulator_;
        
        // in order to display a frame rate
        boost::asio::io_service io_service_;
        boost::asio::steady_timer timer_;
        boost::thread io_service_thread_;
        int frame_count_;
        int frame_rate_;
        
        void set_viewpoint() const;
        void display_point_cloud(const XnUInt8* buf, int size) const;
        void set_environment() const;
        void quit();

        void display_frame_rate();
        void display_polygon();
        void display_balls(void (*near_callback)(void* data, dGeomID o1, dGeomID o2));
        
        void create_mesh();
        void create_balls();
        
        void register_mesh();
        void register_balls();
        
        void set_frame_rate();
};
#endif /* defined(__OpenNiMultiViewer__Viewer__) */
